/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Data::Dumper CDR records.
 * 
 * Copyright (C) 2004, Brian K. West <brian@bkw.org>
 * Copyright (C) 2004, Scott Wolf <admin@aginet.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License.
 *
 */

#include <sys/types.h>
#include <asterisk/channel.h>
#include <asterisk/lock.h>
#include <asterisk/cdr.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/logger.h>
#include <asterisk/utils.h>
#include <asterisk/app.h>

#include "../asterisk.h"
#include "../astconf.h"

#define DUMPER_DIR "/cdr-php-dumper"
#define PHP_APP "/usr/bin/php4 -q"

#define DATE_FORMAT "%Y-%m-%d %T"

#include <stdio.h>
#include <string.h>

#include <stdlib.h>
#include <unistd.h>
#include <time.h>

static char *desc = "PHP Dumper CDR Backend v1.8";

static char *name = "dumper";

static int escape_string(char *src, char *dest, size_t size) {
    int i = 0, j = 0, k = 0;

    for (i = 0; (j < size && src[i] != '\0') ; i++) {
        if (src[i] == '\\' || src[i] == '\'') {
            dest[j++] = '\\';
			k++;
        }
        dest[j++] = src[i];
    }
    dest[j] = '\0';

    return k;
}

static int dumper_log(struct ast_cdr *cdr)
{
	/* Make sure we have a big enough buf */
	FILE *f;
	char buf[8092];
	char buf2[4096];
	char dumper[AST_CONFIG_MAX_PATH];
	char phper[AST_CONFIG_MAX_PATH];
	struct ast_channel *chan;
	struct tm tmstart;
	struct tm tmanswer;
	struct tm tmend;
	struct ast_var_t *variables;
	struct varshead *headp;
	char *var=NULL ,*val=NULL;
	int total = 0;
	char *callerid = NULL;
	size_t size;

	char start[80] = "";
	char answer[80] = "";
	char end[80] = "";

	memset(buf,0,1024);
	
	if((chan = ast_get_channel_by_name_locked(cdr->channel))) {
		memset(buf2,0,4096);
		headp=&chan->varshead;
		snprintf(buf2 + strlen(buf2), sizeof(buf2) - strlen(buf2), "\t'chan_vars' => array(\n");
		AST_LIST_TRAVERSE(headp,variables,entries) {
			if(chan && variables && (var=ast_var_name(variables)) && (val=ast_var_value(variables)) && !ast_strlen_zero(var) && !ast_strlen_zero(val)) {
				if (total == 0 ) {
					snprintf(buf2 + strlen(buf2), sizeof(buf2) - strlen(buf2), "\t\t\t'%s' => '%s'", var, val);
				} else {
					snprintf(buf2 + strlen(buf2), sizeof(buf2) - strlen(buf2), ",\n\t\t\t'%s' => '%s'", var, val);
				}
                if(strlen(buf) >= sizeof(buf2)) {
                    ast_log(LOG_ERROR,"Data Buffer Size Exceeded!\n");
                    break;
                }
                total++;
            } else
                break;
        }
		snprintf(buf2 + strlen(buf2), sizeof(buf2) - strlen(buf2), "\n\t\t\t)\n");
		// is this really needed? 
		ast_mutex_unlock(&chan->lock);
	}

	localtime_r(&cdr->start.tv_sec,&tmstart);
	localtime_r(&cdr->answer.tv_sec,&tmanswer);
	localtime_r(&cdr->end.tv_sec,&tmend);

	strftime(start, sizeof(start), DATE_FORMAT, &tmstart);
	strftime(answer, sizeof(answer), DATE_FORMAT, &tmanswer);
	strftime(end, sizeof(end), DATE_FORMAT, &tmend);

	if (!ast_strlen_zero(cdr->clid)) {
		size = (strlen(cdr->clid) + 1) * 2;
		callerid = alloca(size);
		escape_string(cdr->clid, callerid, size);
	}

	if (!ast_strlen_zero(cdr->accountcode)) {
		snprintf(dumper, sizeof(dumper),"%s/%s/%s.%s", ast_config_AST_LOG_DIR, DUMPER_DIR, cdr->accountcode, cdr->uniqueid);
	} else {
		snprintf(dumper, sizeof(dumper),"%s/%s/Master.%s", ast_config_AST_LOG_DIR, DUMPER_DIR, cdr->uniqueid);
	}
	snprintf(buf, sizeof(buf),
			 "<?php\n\n$x = array(\n\t'clid' => '%s',\n\t'src' => '%s',\n\t'dst' => '%s',\n\t'dcontext' => '%s',\n\t"
			 "'channel' => '%s',\n\t'dstchannel' => '%s',\n\t'lastapp' => '%s',\n\t'lastdata' => '%s',\n\t"
			 "'duration' => '%i',\n\t'billsec' => '%i',\n\t'disposition' => '%i',\n\t'amaflags' => '%i',\n\t"
			 "'accountcode' => '%s',\n\t'start' => '%s',\n\t'answer' => '%s',\n\t'end' => '%s',\n\t"
			 "'flags' => '%i',\n\t'uniqueid' => '%s',\n\t'userfield' => '%s',\n%s"
			 "\t);\n\ninclude(\"save.php\");\n?>\n",
			 callerid, cdr->src, cdr->dst, cdr->dcontext, cdr->channel, cdr->dstchannel, cdr->lastapp,
			 cdr->lastdata, cdr->duration, cdr->billsec, cdr->disposition, cdr->amaflags, cdr->accountcode,
			 start, answer, end, cdr->flags, cdr->uniqueid, cdr->userfield, buf2);

	f = fopen(dumper, "w");
	if (!f)
		return -1;
	fputs(buf, f);
	fflush(f);
	fclose(f);
	sprintf(phper, "nice -n 19 %s %s", PHP_APP, dumper);
        if (ast_safe_system(phper) == -1)
		ast_log(LOG_WARNING, "Execute of %s failed.\n",phper);
	return 0;
}

char *description(void)
{
	return desc;
}

int unload_module(void)
{
	ast_cdr_unregister(name);
	return 0;
}

int load_module(void)
{
	int res;

	res = ast_cdr_register(name, desc, dumper_log);
	if (res) {
		ast_log(LOG_ERROR, "Unable to register PHP Dumper CDR handling v1.6\n");
	}
	return res;
}

int reload(void)
{
	return 0;
}

int usecount(void)
{
	return 0;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
